@@ -1,21 +1,22 @@
 # These functions are
-# Copyright (C) 1998-2017 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2023 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
 
 
-vlm <- function(formula,
-                data = list(),
-                weights = NULL, subset = NULL, na.action = na.fail,
-                prior.weights = NULL,
-                control = vlm.control(...),
-                method = "qr",
-                model = FALSE, x.arg = FALSE, y.arg = TRUE, qr.arg = TRUE,
-                contrasts = NULL,
-                constraints = NULL,
-                extra = NULL, offset = NULL,
-                smart = TRUE, ...) {
+vlm <-
+  function(formula,
+           data = list(),
+           weights = NULL, subset = NULL, na.action = na.fail,
+           prior.weights = NULL,
+           control = vlm.control(...),
+           method = "qr",
+           model = FALSE, x.arg = FALSE, y.arg = TRUE, qr.arg = TRUE,
+           contrasts = NULL,
+           constraints = NULL,
+           extra = NULL, offset = NULL,
+           smart = TRUE, ...) {
   dataname <- as.character(substitute(data))  # "list" if no data=
   function.name <- "vlm"
 
@@ -29,7 +30,8 @@ vlm <- function(formula,
 
 
   mf <- match.call(expand.dots = FALSE)
-  m <- match(c("formula", "data", "subset", "weights", "na.action", "offset"),
+  m <- match(c("formula", "data", "subset", "weights",
+               "na.action", "offset"),
              names(mf), 0)
   mf <- mf[c(1, m)]
   mf$drop.unused.levels <- TRUE
@@ -51,8 +53,9 @@ vlm <- function(formula,
 
   xlev <- .getXlevels(mt, mf)
   y <- model.response(mf, "any")  # model.extract(mf, "response")
-  x <- if (!is.empty.model(mt)) model.matrix(mt, mf, contrasts) else
-       matrix(, NROW(y), 0)
+  x <- if (!is.empty.model(mt))
+         model.matrix(mt, mf, contrasts) else
+         matrix(, NROW(y), 0)
   attr(x, "assign") <- attrassigndefault(x, mt)
 
 
@@ -64,11 +67,11 @@ vlm <- function(formula,
   wz <- model.weights(mf)
 
   y <- as.matrix(y)
-  M <- ncol(as.matrix(y))
+  M <- NCOL(y)
   n <- nrow(x)
   dy <- dimnames(y)
   dy1 <- if (length(dy[[1]])) dy[[1]] else dimnames(mf)[[1]]
-  dy2 <- if (length(dy[[2]])) dy[[2]] else paste("Y", 1:M, sep = "")
+  dy2 <- if (length(dy[[2]])) dy[[2]] else param.names("Y", M)
   dimnames(y) <- list(dy1, dy2)
   predictors.names <- dy2
 
@@ -83,7 +86,7 @@ vlm <- function(formula,
     identity.wts <- TRUE
   } else {
     identity.wts <- FALSE
-    temp <- ncol(as.matrix(wz))
+    temp <- NCOL(wz)
     if (temp < M || temp > M*(M+1)/2)
       stop("input 'w' must have between ", M, " and ", M*(M+1)/2,
            " columns")
@@ -92,12 +95,14 @@ vlm <- function(formula,
 
   control <- control
   Hlist <- process.constraints(constraints, x, M)
-  intercept.only <- ncol(x) == 1 && dimnames(x)[[2]] == "(Intercept)"
+  intercept.only <- ncol(x) == 1 &&
+                    dimnames(x)[[2]] == "(Intercept)"
 
-  fit <- vlm.wfit(xmat = x, zmat = y, Hlist = Hlist, wz = wz, U = NULL,
-                 matrix.out = FALSE, is.vlmX = FALSE,
-                 ResSS = TRUE, qr = qr.arg,
-                 x.ret = TRUE, offset = offset)
+  fit <- vlm.wfit(xmat = x, zmat = y, Hlist = Hlist,
+                  wz = wz, U = NULL,
+                  matrix.out = FALSE, is.vlmX = FALSE,
+                  ResSS = TRUE, qr = qr.arg,
+                  x.ret = TRUE, offset = offset)
 
   ncol.X.vlm <- fit$rank
   fit$R <- fit$qr$qr[1:ncol.X.vlm, 1:ncol.X.vlm, drop = FALSE]
@@ -170,7 +175,8 @@ vlm <- function(formula,
 
   if (length(attr(x, "contrasts")))
       slot(answer, "contrasts") <- attr(x, "contrasts")
-  slot(answer, "na.action") <- if (length(aaa <- attr(mf, "na.action")))
+  slot(answer, "na.action") <-
+    if (length(aaa <- attr(mf, "na.action")))
       list(aaa) else list()
 
   if (length(offset))
